
;TV's server version 3.00
;Copyright (c) 2014, Tycho Veltmeijer.
;All rights reserved.

include 'macro/struct.inc'

FD_READ equ 0x1
FD_WRITE equ 0x2
FD_CLOSE equ 0x20

FD_SETSIZE = 64

struct timeval
 tv_sec dd ?
 tv_usec dd ?
ends

struct fd_set
 fd_count dd ?
 fd_array dd FD_SETSIZE dup (?)
ends

macro FD_SET fd, set
{
 local .bucle1, .fin_bucle1, .fin  
 ;pushad
 mov pebx,fd
 xor pecx,pecx
.bucle1:
 inc pecx
 mov peax,pecx
 shl peax,2
 add peax,set
 add peax,fd_set.fd_count

 cmp dword [peax],ebx
 je  .fin_bucle1    

 cmp ecx,[set+fd_set.fd_count]
 jbe  .bucle1

.fin_bucle1:
 cmp ecx,[set+fd_set.fd_count]
 jbe .fin
 cmp [set+fd_set.fd_count],FD_SETSIZE
 jae .fin  
 mov peax,pecx
 shl peax,2
 add peax,set
 add peax,fd_set.fd_count
 mov dword [peax],ebx
 inc dword [set+fd_set.fd_count]
.fin:

 ;popad
}

macro FD_CLR fd, set
{
 local .bucle1, .cont, .fin   
 ;pushad

 mov pebx,4
 xor pecx,pecx
.bucle1:
 mov peax,pecx
 mul pebx
 add peax,set+fd_set.fd_array

 cmp dword [peax],fd
 jne .cont

 mov ecx,[set+fd_set.fd_count]
 dec pecx
 mov pedi,peax
 add peax,4
 mov pesi,peax
 rep movsd 

 jmp .fin

.cont:
 inc pecx
 cmp ecx,[set+fd_set.fd_count]
 jb  .bucle1

.fin:
 ;popad
}

macro FD_ZERO set
{
 mov [set+fd_set.fd_count],0
}

macro FD_ISSET fd, set
{
 local .bucle1, .cont, .fin  

 pushad
 mov ebx,fd
 mov ecx,[set+fd_set.fd_count]
.find:
 test ecx,ecx
 jz  .notFound

 mov eax,ecx
 shl eax,2    
 add eax,set+fd_set.fd_count

 cmp dword [eax],ebx
 jne  .cont

 mov eax,1
 jmp .fin

.cont:
 dec ecx    
 jmp .find

.notFound:
 xor eax,eax
.fin:	 
 popad

}

;------------------------------------------------------------------------------
;This function will initilize the SOCKET stack
;return: 1 if NO error, else zero
;-----------------------------------------------------------------------------
initilize_socket_api:
xor eax,eax
inc eax
ret

;------------------------------------------------------------------------------
;This function will initilize a listen SOCKET
;default: IP v4 on TCP
;param:
;  flags: 0x1 Use IP v6
;         0x2 Use UDP
;  port:  The port to bind the socket to
;-----------------------------------------------------------------------------
CreateSocket:
call denter
MakeStack

;set the address family
mov ebx,2  ;AF_INET
mov ecx,23 ;AF_INET6

test dword [pebp+psize*2] ,0x1 ;test the flag IPv6
cmovnz ebx,ecx ;set EBX on AF_INET6

;set the type of socket and protocol
mov edi,2 ;SOCK_DGRAM
mov ecx,1 ;SOCK_STREAM

mov esi,17; IPPROTO_UDP
mov edx,6 ;IPPROTO_TCP

test dword [pebp+psize*2] ,0x2 ;test the flag UDP
cmovnz ecx,edi ;set edi on SOCK_DGRAM
cmovnz edx,esi ;set esi on IPPROTO_UDP

;Create the socket
if used is32
  push pedx ;IPPROTO_TCP or IPPROTO_UDP
  push pecx ;SOCK_STREAM or SOCK_DGRAM
  push pebx ;AF_INET or AF_INET6
  argpushsys 2,esp ;arg
  argpushsys 1,1 ;SYS_SOCKET
else
  argpushsys 3,pedx ;IPPROTO_TCP or IPPROTO_UDP
  argpushsys 2,pecx ;SOCK_STREAM or SOCK_DGRAM
  argpushsys 1,pebx ;AF_INET or AF_INET6
end if
linuxCall 41,102  ;sys_socket or 32:sys_socketcall
if used is32
  pop pebx
  add esp,0x8
end if
;check return value on INVALID_SOCKET
inc peax
test peax,peax
jz endCreateSocket
dec peax

mov pedx,peax

;size parameter
mov pesi,0x10

;make room on stack for sockaddr_in structure
sub pesp,0x10

test dword [pebp+psize*2] ,0x1 ;test the flag IPv6
jz createIPv4Structure

;add extra size on stack to create sockaddr_in6 structure
sub pesp,0x10
add pesi,0x10

createIPv4Structure:
mov pedi,pesp ;pointer to sockaddr_in(6)

;Zero the structure
xor eax,eax
mov ecx,esi ;size of the structure
rep stosb


;pointer to sockaddr_in(6)
mov pedi,pesp ;pointer to sockaddr_in(6)

;fill in member family
mov word [pedi],bx ;set family address


;Fill in member port
mov eax,dword [pebp+psize*3] ;get Port from stack
xchg al, ah ;convert to big-endian
mov word [pedi+2],ax ;fill in the port number

mov ebx,edx ;save socket on EBX

if used is32
  push pesi ;Size of sockaddr_in structure
  push pedi ;Pointer to sockaddr_in structure
  push pebx ;socket
  argpushsys 2,esp ;arg
  argpushsys 1,2 ;sys_bind
else
  argpushsys 3,pesi ;Size of sockaddr_in structure
  argpushsys 2,pedi ;Pointer to sockaddr_in structure
  argpushsys 1,pebx ;socket
end if
linuxCall 49,102  ;sys_bind or 32:sys_socketcall
if used is32
  pop pebx
  add esp,0x8
end if

inc eax
test eax,eax
jz errCreateSocket

test dword [pebp+psize*2] ,0x2 ;test the flag UDP
jnz dontListen

if used is32
  push 0x7FFFFfff ;backlog
  push pebx	   ;socket
  argpushsys 2,esp ;arg
  argpushsys 1,4	   ;sys_listen
else
  argpushsys 2,0x7FFFFfff ;backlog
  argpushsys 1,pebx	   ;socket
end if
linuxCall 50,102  ;sys_listen or 32:sys_socketcall
if used is32
  pop pebx
  add esp,0x4
end if

inc eax
test eax,eax
jz errCreateSocket

dontListen:
mov peax,pebx

endCreateSocket:
call dleave
ret psize*2

;-
errCreateSocket:
argpushsys 1,pebx
linuxCall 3,6  ;sys_close

xor eax,eax
jmp endCreateSocket

;------------------------------------------------------------------------------
;This function will wait for a client to connect
;param:
;  socket: the socket to listen to
;return: -1 or a socket descriptor
;-----------------------------------------------------------------------------
Accept:
call denter
MakeStack

recallAccept:
if used is32
  push 0
  push 0
  push pdword [pebp+psize*2] ;Socket
  argpushsys 2,esp ;arg
  argpushsys 1,5 ;sys_accept
else
  argpushsys 3,0
  argpushsys 2,0
  argpushsys 1,pdword [pebp+psize*2] ;Socket
end if
linuxCall 43,102  ;sys_accept
if used is32
  add esp,0xC
end if

mov ecx,eax
inc ecx
test ecx,ecx
jnz endAccept

mov ebx,dword [errno]

push peax

call denter

;sleep 1 milisecond
push 1000000
push 0
mov pebx,pesp

argpushsys 2,0
argpushsys 1,pebx
linuxCall 35,162 ;sys_nanosleep

add pesp,psize*2

call dleave

pop peax

cmp ebx,10054 ;WSAECONNRESET
je recallAccept

cmp ebx,10004 ;WSAEINTR
je recallAccept

cmp ebx,10024 ;WSAEMFILE
je recallAccept

xor eax,eax
dec eax

endAccept:
call dleave
ret psize

;------------------------------------------------------------------------------
;This function will create a event for close, write and read events for a socket
;param:
;  socket: the socket to create the event on
;return: -1 or a event descriptor
;-----------------------------------------------------------------------------
CreateSocketEvent:
call denter
;--MakeStack

;--call [WSACreateEvent]
;--mov pebx,peax

;--argpush 3,FD_READ or FD_WRITE or FD_CLOSE  ;trigger
;--argpush 2,peax                             ;event
;--argpush 1,pdword [pebp+psize*2]            ;SOCKET
;--call [WSAEventSelect]

;--mov peax,pebx

;-argpushsys 3,00004000o ; O_NONBLOCK
;-argpushsys 2,4 ;F_SETFL
;-argpushsys 1,pdword [pebp+psize*2] ;socket
;-linuxCall 72,55 ;sys_fcntl

call dleave

xor eax, eax
inc eax
ret psize

;------------------------------------------------------------------------------
;This function will close an event
;param:
;  event: event descriptor
;return: boolean
;-----------------------------------------------------------------------------
CloseSocketEvent:
;--call denter
;--MakeStack

;--argpush 1,pdword [pebp+psize*2]            ;event
;--call [WSACloseEvent]

;--call dleave
ret psize

;------------------------------------------------------------------------------
;This function will close a socket, no shutdown or annything
;param:
;  Socket
;-----------------------------------------------------------------------------
CloseSocket:
CloseGeneralSocket:
call denter
MakeStack

mov pedi,pdword [pebp+psize*2]
jmp _closeSocketUnsafe

;------------------------------------------------------------------------------
;This function will send and receive remaining data and finaly close the socket
;param:
;  Socket
;-----------------------------------------------------------------------------
SaflyCloseTCPSocket:
call denter
MakeStack

mov pedi,pdword [pebp+psize*2] ;SOCKET

if used is32
  push 1;(=SD_SEND)          ;SD_SEND
  push pedi		   ;SOCKET
  argpushsys 2,esp ;arg
  argpushsys 1,13	   ;sys_shutdown
else
  argpushsys 2,1;(=SD_SEND)          ;SD_SEND
  argpushsys 1,pedi		   ;SOCKET
end if
linuxCall 48,102  ;sys_shutdown
if used is32
  add esp,0x8
end if

sub pesp,80
if used is64
   mov r12,pesp
end if

MakeStack

recvRemainingData:
  if used is32
     mov pecx,pesp
  end if

  push pebp

if used is32
  push 0
  push 0
  push 0		     ;flags
  push 80		     ;length of buffer
  push ecx
  push pedi		  ;SOCKET
  argpushsys 2,esp ;arg
  argpushsys 1,10		  ;sys_recv
else
  argpushsys 6,0
  argpushsys 5,0
  argpushsys 4,0		     ;flags
  argpushsys 3,80		     ;length of buffer
  argpushsys 2,r12		     ;pointer to the buffer
  argpushsys 1,pedi		  ;SOCKET
end if
  linuxCall 45,102	;sys_recvfrom
if used is32
  add esp,0x18
end if

  pop pebp

test eax,eax
jz saflyCloseSocket

if used is64
  cmp rax,0xFFFFFFFFFFFFFF95
else
  cmp eax,0xFFFFFF95
end if
jne recvRemainingData

saflyCloseSocket:

_closeSocketUnsafe:
argpushsys 1,pedi
linuxCall 3,6  ;sys_close

call dleave
ret psize

;------------------------------------------------------------------------------
;This function will send data
;param:
;  1:Socket
;  2:flags
;  3:buf
;  4:&len
;
;flags: RSF_RETURN_ON_ERROR (0x1) Return on error, do not try to resolve the error
;
;Return: CE_NOERR (0x0)
;        CE_NO_BUFFER_AVAILABLE (0x1)
;        CE_CONNECTION_CLOSED (0x2)
;        CE_UNKNOWN_ERROR (0x3)
;Return on &len: The data that is send (on success)
;-----------------------------------------------------------------------------
SendSocketData:
call denter
MakeStack

;(R/E)BX = SOCKET
mov pebx,pdword [pebp+psize*2]
;(R/E)SI = buf
mov pesi,pdword [pebp+psize*4]
;(R/E)DI = &len
mov pedi,pdword [pebp+psize*5]

retrySendSocketData:
push pebp
push pesi
push pedi

if used is32
  push 0
  push 0
  push 0
  push dword [pedi];len
  push pesi	   ;Buff
  push pebx	   ;SOCKET
  argpushsys 2,esp ;arg
  argpushsys 1,9   ;send
else
  argpushsys 6,0
  argpushsys 5,0
  argpushsys 4,0
  mov edx,dword [pedi]	;len;argpushsys 3,dword [pedi]  ;len
  argpushsys 2,pesi	   ;Buff
  argpushsys 1,pebx	   ;SOCKET
end if
linuxCall 44,102 ;sys_sendto
if used is32
  pop pebx
  add esp,0x14
end if

pop pedi
pop pesi
pop pebp

test eax,eax
jz sendRetZero

inc eax
test eax,eax
jnz successSendSocketData

mov ecx,CE_UNKNOWN_ERROR

;if RSF_RETURN_ON_ERROR is set we do not resolve any error
test pdword [pebp+psize*3],RSF_RETURN_ON_ERROR
cmovnz eax,ecx ;set error code and return if the flag is set
jnz endSendSocketData

;Try to resolve the error

sendRetZero:
;Get the last Socket error
;WSAGetLastError is a redirect to GetLastError(), Sleep() will set this to "No Error"
mov eax,dword [errno]
push peax

;Sleep one milisecond so we do not eat CPU time
push peax

call denter

;sleep 1 milisecond
push 1000000
push 0
mov pebx,pesp

argpushsys 2,0
argpushsys 1,pebx
linuxCall 35,162 ;sys_nanosleep

add pesp,psize*2

call dleave

pop peax

pop peax

cmp eax,105 ;ENOBUFS
je retrySendSocketData

;cmp eax,10040 ;WSAEMSGSIZE
;je retrySendData

cmp eax,14 ;EFAULT
je retrySendSocketData

;--cmp eax,10036 ;WSAEINPROGRESS
;--je retrySendSocketData

cmp eax,11;EWOULDBLOCK
je retrySendSocketData

;--cmp eax,10065;WSAEHOSTUNREACH
;--je retrySendSocketData

cmp eax,4;WSAEINTR
je retrySendSocketData

cmp eax,13;EACCES
je GetAcces

mov eax,CE_CONNECTION_CLOSED
jmp endSendSocketData

GetAcces:
call denter

;Get acces to send data
if used is32
  push 0
  push 0
  push 0x20   ;SO_BROADCAST
  push 0xFFFF ;SOL_SOCKET
  push pebx   ;SOCKET
  argpushsys 2,esp ;arg
  argpushsys 1,14   ;setsockopt
else
  argpushsys 5,0
  argpushsys 4,0
  argpushsys 3,0x20   ;SO_BROADCAST
  argpushsys 2,0xFFFF ;SOL_SOCKET
  argpushsys 1,pebx   ;SOCKET
end if
linuxCall 54,102 ;sys_setsockopt
if used is32
  pop pebx
  add esp,0x10
end if

call dleave

jmp retrySendSocketData

successSendSocketData:
dec eax
mov dword [pedi],eax

xor eax,eax ;no error

endSendSocketData:
call dleave
ret 4*psize

;------------------------------------------------------------------------------
;This function will wait until the socket can be read OR when a timeout happend
;param:
;  1:Socket
;  2:timeout

;Return 0 or -1 on failure
;-----------------------------------------------------------------------------
WaitForRead:
call denter

;create fd_set
sub pesp,sizeof.fd_set
mov pesi,pesp
mov pebx,pdword [pebp+psize*2]

FD_ZERO pesp
FD_SET pdword [pebp+psize*2],pesp

mov pesi,pesp

mov peax,pdword [pebp+psize*3]	;miliseconds

xor edx,edx
mov ecx,1000
div pecx

if used is32
  push peax ;push usec
else
  push 0
  mov dword [pesp+0x4],eax ;push usec
end if


mov eax,edx ;res

mov ecx,1000
mul peax

;xchg peax,pdword [pesp]
if used is32
  push peax ;push sec
else
  mov dword [pesp],eax ;push sec
end if
mov pedi,pesp

;--;create fd_set
;--mov peax,pdword [pebp+psize*2]
;--;push peax
;--push peax
;--push 1
;--mov pesi,pesp

argpushsys 5,pedi		      ;timeout
argpushsys 2,pesi		      ;readfds
argpushsys 4,0			      ;exceptfds
argpushsys 3,0			      ;writefds
mov peax,pdword [pebp+psize*2]
inc peax
argpushsys 1,peax		      ;nfds
linuxCall 23,142 ;sys_select

call dleave
ret psize*2


;------------------------------------------------------------------------------
;This function return the time the system is running in miliseconds
;param:

;Return runtime in miliseconds
;-----------------------------------------------------------------------------
getTickCount:
call denter

push 0
push 0

argpushsys 2,0
argpushsys 1,pesp
linuxCall 96,78 ;sys_gettimeofday

mov eax,dword [pesp]  ;seconds
xor edx,edx
mov ecx,1000
mul pecx

xchg peax,pdword [pesp+0x4] ;usec = sec

mov ecx,1000
div pecx

add peax,pdword [pesp+0x4]

add pesp,psize


;--push 0
;-push 0

;--argpushsys 2,pesp
;--argpushsys 1,1 ;CLOCK_MONOTONIC
;--linuxCall 228,265 ;sys_clock_gettime

;--pop peax  ;seconds
;--xor edx,edx
;--mov ecx,1000
;--mul peax

;--xchg peax,pdword [pesp]

;--mov ecx,1000000
;--div pecx

;--add peax,pdword [pesp]

call dleave
ret

;------------------------------------------------------------------------------
;This function will receive data
;param:
;  1:Socket
;  2:flags
;  3:timeout
;  4:buf
;  5:&len
;
;flags: RSF_RETURN_ON_ERROR (0x1) Return on error, do not try to resolve the error
;       RSF_PEEK_DATA (0x2) Leave the data on the internal buffer
;       RSF_RETURN_ON_FIRST_SUCCESSFUL_ATTEMPT (0x4) return afther one successful try receiving data from the client, do not check if 'size' is received
;
;Return: CE_NOERR (0x0)
;        CE_NO_BUFFER_AVAILABLE (0x1)
;        CE_CONNECTION_CLOSED (0x2)
;        CE_UNKNOWN_ERROR (0x3)

;Return on &len: The data that is received (on success)
;-----------------------------------------------------------------------------
RecvSocketData:
call denter
MakeStack

;(R/E)BX = SOCKET
mov pebx,pdword [pebp+psize*2]
;(R/E)DI = &len
mov pedi,pdword [pebp+psize*6]

; we cannot use MSG_WAITALL (0x100) since we need to keep track of the timeout value

;get flags
mov esi,dword [pebp+psize*3]

call getTickCount

;r12 or ECX = start time of this function
;r13 or EDX = received len
if used is32
   mov ecx,eax
   xor edx,edx
else
   mov r12d,eax
   xor r13d,r13d
end if

;loopReceiveDataWithSaveTickCount:
if used is32
   push ecx	 ;save tickcount
end if

xor eax,eax

mov ecx,esi
and ecx,RSF_PEEK_DATA	 ;since RSF_PEEK_DATA == MSG_PEEK we can use it directly for recv

if used is32
   cmovnz edx,eax	;zero len if RSF_PEEK_DATA is set
   push edx		;save len value
else
   cmovnz r13d,eax	;zero received buffer if RSF_PEEK_DATA is set
end if

loopReceiveData:
mov ecx,esi
and ecx,RSF_PEEK_DATA	 ;since RSF_PEEK_DATA == MSG_PEEK

mov eax,dword [pedi]	;EAX = len

if used is32
   pop edx
   jnz peekFlagIsSet
     sub eax,edx	  ;len - received buffer
   peekFlagIsSet:
   push edx
else
   jnz peekFlagIsSet
     sub eax,r13d	  ;len - received buffer
   peekFlagIsSet:
end if

mov edx,eax

push pesi

mov pesi,pdword [pebp+psize*5]

or pecx,0x40;MSG_DONTWAIT

;mov dword [errno],0

call denter

if used is32
  push 0
  push 0
  push pecx		     ;flags
  push pedx		     ;length of buffer
  push pesi		     ;buff
  push pebx		  ;SOCKET
  argpushsys 2,esp ;arg
  argpushsys 1,10		  ;sys_recv
else
  argpushsys 6,0			;
  argpushsys 5,0			;
  argpushsys 4,pecx			;flags
  argpushsys 3,pedx			;len
  argpushsys 2,pesi			;buff
  argpushsys 1,pebx			;SOCKET
end if
linuxCall 45,102 ;sys_recvfrom
if used is32
  pop pebx
  add esp,0x14
end if

call dleave
pop pesi

test eax,eax
jz recvRetZero

;--mov ecx,CE_UNKNOWN_ERROR

;check on INVALID_SOCKET
;inc eax ;incement EAX,  -1 will be 0
;test eax,eax
if used is32
  cmp eax,0xFFFFFFF5
else
  cmp rax,0xFFFFFFFFFFFFFFF5
end if
;--cmovz eax,ecx
;--jz endRecvSocketData

;Since a Recv error cannot be resolved, return an error
jnz noErrorRecv

;An error accord, return if RSF_RETURN_ON_ERROR is set
mov ecx,CE_UNKNOWN_ERROR

test esi,RSF_RETURN_ON_ERROR
cmovnz eax,ecx
jnz endRecvSocketData

recvRetZero:
;Get the last Socket error
lea peax,dword [errno]
mov eax,dword [peax]

push peax

call denter

;sleep 1 milisecond
push 1000000
push 0
mov pebx,pesp

argpushsys 2,0
argpushsys 1,pebx
linuxCall 35,162 ;sys_nanosleep

add pesp,psize*2

call dleave

pop peax

cmp eax,11;EWOULDBLOCK
je checkOnTimeout; loopReceiveData

mov eax,CE_UNKNOWN_ERROR
jmp endRecvSocketData

noErrorRecv:
;--dec eax  ;restore EAX

test esi,RSF_RETURN_ON_FIRST_SUCCESSFUL_ATTEMPT
jz dontReturnRecvSocketData

test eax,eax ;if no data received
jz checkOnTimeout

mov dword [pedi],eax  ;set &len
xor eax,eax ;CE_NO_ERROR
jmp endRecvSocketData

dontReturnRecvSocketData:
xor ecx,ecx  ;CE_NOERROR

;if PEEK flag is set, don't do ADD, do a move operation
test esi,RSF_PEEK_DATA

if used is32
   pop edx	   ;pop received len
   cmovnz edx,eax  ;move data if PEEK flag is set (the next time it will give the same data, again and again and again)_
   jnz dontAddSizeSinceItIsPeek
   add edx,eax	   ;add the received len from this try
   dontAddSizeSinceItIsPeek:
   push edx		;save len value
   ;compair if the buffer is full
   cmp edx,dword [pedi]
else
   cmovnz r13d,eax  ;move data if PEEK flag is set (the next time it will give the same data, again and again and again)_
   jnz dontAddSizeSinceItIsPeek
   add r13d,eax    ;add the received len from this try
   dontAddSizeSinceItIsPeek:
   ;compair if the buffer is full
   cmp r13d,dword [pedi]
end if

cmovz eax,ecx  ;Set CE_NOERROR    ;cmovnc
jz endRecvSocketData		  ;jae

test esi,RSF_PEEK_DATA	  ;test on RSF_PEEK_DATA
jnz checkOnTimeout	   ;jump if flag is set

;move the buffer +received bytes
add pdword [pebp+psize*5],peax

checkOnTimeout:
;check if we need to check on timeout
mov edx,dword [pebp+psize*4]  ;get timeout value
test edx,edx
jz loopReceiveData ;no Timout

call getTickCount

if used is32
   sub eax,dword [esp+0x4] ;restore start time of this function and substract it from current time
else
   sub eax,r12d
end if

cmp eax,dword [pebp+psize*4] ;check with TimeOut value
jb loopReceiveData ;not yet timed out

_TIMEOUT:
mov eax,CE_TIMEOUT ;err code

if used is32
   pop edx		 ;pop received len
   mov dword [pedi],edx  ;set &len value
else
   mov dword [pedi],r13d ;set &len value
end if

endRecvSocketData:
call dleave
ret psize*5
