
;TV's server version 3.00
;Copyright (c) 2014, Tycho Veltmeijer.
;All rights reserved.


if used getMimeTypeFromExtension
valueNameContentType db 'Content Type',0

;------------------------------------------------------------------------------
;This function will get a MIM
;input:
;    (R/E)AX:pointer with extension
;output:
;    (R/E)AX=> pointer with mime-type
;    (R/E)CX=> size of pointer
;------------------------------------------------------------------------------
getMimeTypeFromExtension:
push pedx

push valueNameContentType  ;valueName
push 0			   ;SubSubSubSubKey
push 0			   ;SubSubSubKey
push 0			   ;SubSubKey
push peax		   ;file extension
mov eax,HKEY_CLASSES_ROOT
push peax		   ;hKey
call ReadRegEx
test edx,edx
jz endGetMimeTypeFromExtension

;ReadRegEx failed, clear allocated memory
push peax
call FreeMemLocation

xor eax,eax  ;return zero

endGetMimeTypeFromExtension:
pop pedx
ret
end if

;------------------------------------------------------------------------------
;This function will open a file
;input:
;    1:pointer with filename
;    2:flags:
;       0x0 READ
;       0x1 WRITE&READ
;       0x2 CREATE_FILE

;output:
;    (R/E)AX=> filehandle or -1 on failure
;------------------------------------------------------------------------------
if used openFileHandle
openFileHandle:
call denter
MakeStack

mov edi,FILE_SHARE_READ ;dwShareMode

mov edx,0x80000000  ;GENERIC_READ
test pdword [pebp+psize*3],0x1 ;WRITE&READ
jz noWriteRightsRequired

  or edx,0x40000000 ;GENERIC_WRITE
  xor edi,edi ;dwShareMode

noWriteRightsRequired:
mov esi,OPEN_EXISTING ;ESI => parameter CreationDisposition
test pdword [pebp+psize*3],0x2 ;CREATE_FILE&READ
jz noFileCreate

  or esi,OPEN_ALWAYS

noFileCreate:
argpush 7,0			;hTemplateFile
argpush 6,0			;dwFlagsAndAttributes
argpush 5,pesi			;dwCreationDisposition
argpush 4,0			;lpSecurityAttributes
argpush 3,pedi			;dwShareMode
argpush 2,edx			;dwDesiredAccess
argpush 1,pdword [pebp+psize*2] ;filename
call [CreateFileA]

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will return the size of a file (with limit of 4GiB on x86)
;input:
;    1:file handle

;output:
;    (R/E)AX => low integer with size

;------------------------------------------------------------------------------
if used getfilesize | (used getfilesizeLongLong & used is64)
getfilesize:
if used getfilesizeLongLong & used is64
  getfilesizeLongLong:
end if
call denter
MakeStack

if used is32
  ;renember: we cant use GetFileSizeEx in x86 if we want to support Windows NT
  push 0		    ;high word is not needed here (call getfilesizeLongLong in x86 mode for long long support)
  push dword [pebp+psize*2] ;file handle
  call [GetFileSize]
else
  ;x64 is supported since XP and GetFileSizeEx is supported since 2000, so no problem here.
  lea rdx,qword [pesp+psize*3] ;free spot on the stack for the return long long
  mov rcx,qword [pebp+psize*2] ;file handle
  call [GetFileSizeEx]

  mov rax,qword [pesp+psize*3] ;set file size on RAX
end if

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will return the size of the file
;input x86:
;    1:file handle
;    2:pointer to long long
;input x64:
;    1:file handle

;output x86:
;    EAX => low integer with size
;    [2] => high integer with size

;output x64:
;    RAX => size of file
;------------------------------------------------------------------------------
if used getfilesizeLongLong & used is32
getfilesizeLongLong:
call denter

mov edi,dword [ebp+psize*3]  ;long long
add edi,4		     ;high part of the long long

push edi		 ;the return pointer for the higher part of the size
push dword [ebp+psize*2] ;file handle
call [GetFileSize]

sub edi,4 ;low part of the long long
stosd

call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will read out a file
;input:
;    1:file handle
;    2:pointer to buffer
;    3:size to read

;output:
;    EAX => size read
;------------------------------------------------------------------------------
if used readfile
readfile:
call denter
MakeStack

if used is32
  push 0 ;make space for the return read size
  mov edx,esp
else
  lea r9,dword [rsp+psize*6] ;argument nNumberOfBytesToRead => pointer to free space on stack
end if

argpush 5,0  ;lpOverlapped
if used is32
  push edx ;nNumberOfBytesToRead
else
  ;argpush 4,r9 (already done)
end if
argpush 3,pdword [pebp+psize*4] ;size to read
argpush 2,pdword [pebp+psize*3] ;buffer
argpush 1,pdword [pebp+psize*2] ;file handle
call [ReadFile]

if used is32
  pop peax ;pop the read size
else
  mov eax,dword [rsp+psize*6] ;the read size
end if

call dleave
ret psize*3
end if

;------------------------------------------------------------------------------
;This function will read out a file
;input:
;    1:file handle
;    2:(x64)filepointer to seek to (x86)pointer to a long long with a filepointer to seek to

;output:
;    EAX => return value SetFilePointer (the low-order dword of the current file pointer
;------------------------------------------------------------------------------
if used seekToPointerFile
seekToPointerFile:
call denter
MakeStack

mov peax,pdword [pebp+psize*3] ;(x64)pointer to seek to (x86)pointer to a long long to seek to

if used is32
  ;SetFilePointerEx is avaidable since win 2000, so call SetFilePointer in 32-bit to support winNT
  push FILE_BEGIN	     ;seek from begining
  push dword [peax+psize]    ;high part of the long long
  push dword [peax]	     ;low part of the long long
  push dword [pebp+psize*2]  ;file handle
  call [SetFilePointer]
else
  ;x64 is avaidable since Win XP, so we are able to use SetFilePointerEx
  argpush 4,FILE_BEGIN		  ;seek from begining
  argpush 3,0			  ;high part of the long long
  argpush 2,rax 		  ;long long
  argpush 1,pdword [pebp+psize*2] ;file handle
  call [SetFilePointerEx]
end if


call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function will close a file handle
;input:
;    1:file handle

;output:
;    EAX => file handle
;------------------------------------------------------------------------------
if used closefile
closefile:
call denter
MakeStack

argpush 1,pdword [pebp+psize*2]
call [CloseHandle]

mov peax,pdword [pebp+psize*2]

call dleave
ret psize
end if

;------------------------------------------------------------------------------
;This function will check if a directorie exist
;input:
;    1:path

;output:
;       eax:0 (no directoty) 0x10:directory
;flags: Z: No directory  z: directory
;------------------------------------------------------------------------------
if used isDirectory
isDirectory:
call denter
MakeStack

argpush 1,pdword [pebp+psize*2] ;path
call [GetFileAttributesA]

call dleave

cmp eax,-1
jz endIsDirectory

and eax,FILE_ATTRIBUTE_DIRECTORY

endIsDirectory:
ret psize
end if

;------------------------------------------------------------------------------
;This function start (and return an handle) to enumerate file(s) and directories in a directory
;input:
;    1:path
;    2: return structure (total: 4+(7*2)+4+8+260=290, rounded by 8=296):
;       Atributes (dword):
;           0x10 Directory
;       TIMEDATE structure (7*2 + 4)
;       Filesize (long long, no pointer)
;       Filename[260]



;output:
;       eax:HANDLE or zero
;------------------------------------------------------------------------------
if used startDirectoryEnumeration
directoryWildFlag db '*.*',0

startDirectoryEnumeration:
call denter
sub pesp,11*4 + 264 + 16 + 4 ;create an _WIN32_FIND_DATA on stack (+4 for rouning up)
mov pesi,pesp ;(R/E)SI => pointer to _WIN32_FIND_DATA structure

;zero structure
mov pedi,pesp ;pointer to _WIN32_FIND_DATA structure
if used is32
  mov ecx,82
  rep stosd
else
  mov ecx,41
  rep stosq
end if

MakeStack

push 0 ;flags
push directoryWildFlag
push pdword [pebp+psize*2] ;path
call combineStrings
mov pedi,peax ;(R/E)SI => pointer to path + "*.*"

argpush 2,pesi	   ;LPWIN32_FIND_DATA
argpush 1,pedi	   ;path
call [FindFirstFileA]
mov pebx,peax  ;(R/E)BX => handle

;free temp path string
push pedi
call FreeMemLocation

inc pebx    ;-1 + 1 = 0
test pebx,pebx
cmovz eax,edx
jz endStartDirectoryEnumeration ;return if invalid handle
dec pebx  ;restore handle

mov pedi,pdword [pebp+psize*3] ;return structure

movsd ;copy atributes
add pesi,8*2 ;skip ftCreationTime and LastAccessTime

stosw ;skip first two bytes

;convert last write time
argpush 2,pedi ;SYSTEMTIME structure
argpush 1,pesi ;last write time
call [FileTimeToSystemTime]

;convert year wordt to dword
xor ax,ax
xchg ax,word [pedi]
mov word [pedi-2],ax

add pesi,8 ;skip ftLastWriteTime
add pedi,(8*2) ;skip SYSTEMTIME structure

;swap high and low
lodsd ;get high dword of the size member
movsd ;copy the low dword of the size member
stosd ;store tthe high dword

add pesi,8 ;skip reserved values

if used is32
  mov ecx,65 ;=260 / 4
  rep movsd ;copy the filename
else
  mov ecx,32 ;=256 / 4
  rep movsq ;copy the first 256 bytes of the filename
  movsd     ;copy last 4 bytes
end if


mov peax,pebx ;return handle

endStartDirectoryEnumeration:
call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function return next next file in the directory
;input:
;    1:handle
;    2: return structure (total: 4+(7*2)+4+8+260=290, rounded by 8=296):
;       Atributes (dword):
;           0x10 Directory
;       TIMEDATE structure (7*2 + 4)
;       Filesize (long long, no pointer)
;       Filename[260]



;output:
;       eax:true of false
;------------------------------------------------------------------------------
if used nextDirectoryEnumeration
nextDirectoryEnumeration:
call denter
sub pesp,(11*4) + 264 + 16+ 4 ;create an _WIN32_FIND_DATA on stack (+4 for rouning up)
mov pesi,pesp ;pointer to _WIN32_FIND_DATA structure

;zero structure
mov pedi,pesp ;pointer to _WIN32_FIND_DATA structure
if used is32
  mov ecx,82
  rep stosd
else
  mov ecx,41
  rep stosq
end if

MakeStack

argpush 2,pesi			    ;LPWIN32_FIND_DATA
argpush 1,pdword [pebp+psize*2]     ;handle
call [FindNextFileA]

test peax,peax
jz endNextDirectoryEnumeration ;return if invalid handle

mov pedi,pdword [pebp+psize*3] ;return structure

movsd ;copy atributes
add pesi,8*2 ;skip ftCreationTime and LastAccessTime

stosw ;skip first two bytes

;convert last write time
argpush 2,pedi ;SYSTEMTIME structure
argpush 1,pesi ;last write time
call [FileTimeToSystemTime]

;convert year wordt to dword
xor ax,ax
xchg ax,word [pedi]
mov word [pedi-2],ax

add pesi,8 ;skip ftLastWriteTime
add pedi,(8*2) ;skip SYSTEMTIME structure

;swap high and low
lodsd ;get high dword of the size member
movsd ;copy the low dword of the size member
stosd ;store tthe high dword

add pesi,8 ;skip reserved values

if used is32
  mov ecx,65 ;=260 / 4
  rep movsd ;copy the filename
else
  mov ecx,32 ;=256 / 4
  rep movsq ;copy the first 256 bytes of the filename
  movsd     ;copy last 4 bytes
end if

xor eax,eax
inc eax

endNextDirectoryEnumeration:
call dleave
ret psize*2
end if

;------------------------------------------------------------------------------
;This function close the handle of the enumeration
;input:
;    1:handle


;output:
;       none
;------------------------------------------------------------------------------
if used closeDirectoryEnumeration
closeDirectoryEnumeration:
call denter
MakeStack

argpush 1,pdword [pebp+psize*2]
call [FindClose]

call dleave
ret psize
end if